@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Union
+from typing import Union, Dict, Any
 from typing import Type
 from typing import Optional
 
@@ -33,27 +33,25 @@ from libcloud.utils.py3 import httplib
 from libcloud.utils.py3 import urlparse
 from libcloud.utils.py3 import urlencode
 
-from libcloud.utils.misc import lowercase_keys, retry
+from libcloud.utils.misc import lowercase_keys
+from libcloud.utils.retry import Retry
 from libcloud.common.exceptions import exception_from_message
 from libcloud.common.types import LibcloudError, MalformedResponseError
 from libcloud.http import LibcloudConnection, HttpLibResponseProxy
 
 __all__ = [
-    'RETRY_FAILED_HTTP_REQUESTS',
-
-    'BaseDriver',
-
-    'Connection',
-    'PollingConnection',
-    'ConnectionKey',
-    'ConnectionUserAndKey',
-    'CertificateConnection',
-
-    'Response',
-    'HTTPResponse',
-    'JsonResponse',
-    'XmlResponse',
-    'RawResponse'
+    "RETRY_FAILED_HTTP_REQUESTS",
+    "BaseDriver",
+    "Connection",
+    "PollingConnection",
+    "ConnectionKey",
+    "ConnectionUserAndKey",
+    "CertificateConnection",
+    "Response",
+    "HTTPResponse",
+    "JsonResponse",
+    "XmlResponse",
+    "RawResponse",
 ]
 
 # Module level variable indicates if the failed HTTP requests should be retried
@@ -80,18 +78,18 @@ class LazyObject(object):
                 pass
 
             def __getattribute__(self, attr):
-                lazy_obj = object.__getattribute__(self, '_get_lazy_obj')()
+                lazy_obj = object.__getattribute__(self, "_get_lazy_obj")()
                 return getattr(lazy_obj, attr)
 
             def __setattr__(self, attr, value):
-                lazy_obj = object.__getattribute__(self, '_get_lazy_obj')()
+                lazy_obj = object.__getattribute__(self, "_get_lazy_obj")()
                 setattr(lazy_obj, attr, value)
 
             def _get_lazy_obj(self):
-                lazy_obj = object.__getattribute__(self, '_lazy_obj')
+                lazy_obj = object.__getattribute__(self, "_lazy_obj")
                 if lazy_obj is None:
                     lazy_obj = cls(*lazy_init_args, **lazy_init_kwargs)
-                    object.__setattr__(self, '_lazy_obj', lazy_obj)
+                    object.__setattr__(self, "_lazy_obj", lazy_obj)
                 return lazy_obj
 
         return Proxy()
@@ -110,11 +108,11 @@ class HTTPResponse(httplib.HTTPResponse):
     # This applies the behaviour from 2.7, fixing the hangs.
     def read(self, amt=None):
         if self.fp is None:
-            return ''
+            return ""
 
-        if self._method == 'HEAD':
+        if self._method == "HEAD":
             self.close()
-            return ''
+            return ""
 
         return httplib.HTTPResponse.read(self, amt)
 
@@ -157,14 +155,16 @@ class Response(object):
         self.request = response.request
         self.iter_content = response.iter_content
 
-        self.body = response.text.strip() \
-            if response.text is not None and hasattr(response.text, 'strip') \
-            else ''
+        self.body = (
+            response.text.strip()
+            if response.text is not None and hasattr(response.text, "strip")
+            else ""
+        )
 
         if not self.success():
-            raise exception_from_message(code=self.status,
-                                         message=self.parse_error(),
-                                         headers=self.headers)
+            raise exception_from_message(
+                code=self.status, message=self.parse_error(), headers=self.headers
+            )
 
         self.object = self.parse_body()
 
@@ -177,7 +177,7 @@ class Response(object):
         :return: Parsed body.
         :rtype: ``str``
         """
-        return self.body if self.body is not None else ''
+        return self.body if self.body is not None else ""
 
     def parse_error(self):
         """
@@ -202,8 +202,14 @@ class Response(object):
         """
         # pylint: disable=E1101
         import requests
-        return self.status in [requests.codes.ok, requests.codes.created,
-                               httplib.OK, httplib.CREATED, httplib.ACCEPTED]
+
+        return self.status in [
+            requests.codes.ok,
+            requests.codes.created,
+            httplib.OK,
+            httplib.CREATED,
+            httplib.ACCEPTED,
+        ]
 
 
 class JsonResponse(Response):
@@ -219,9 +225,8 @@ class JsonResponse(Response):
             body = json.loads(self.body)
         except Exception:
             raise MalformedResponseError(
-                'Failed to parse JSON',
-                body=self.body,
-                driver=self.connection.driver)
+                "Failed to parse JSON", body=self.body, driver=self.connection.driver
+            )
         return body
 
     parse_error = parse_body
@@ -241,12 +246,13 @@ class XmlResponse(Response):
                 body = ET.XML(self.body)
             except ValueError:
                 # lxml wants a bytes and tests are basically hard-coded to str
-                body = ET.XML(self.body.encode('utf-8'))
+                body = ET.XML(self.body.encode("utf-8"))
         except Exception:
-            raise MalformedResponseError('Failed to parse XML',
-                                         body=self.body,
-                                         driver=self.connection.driver)
+            raise MalformedResponseError(
+                "Failed to parse XML", body=self.body, driver=self.connection.driver
+            )
         return body
+
     parse_error = parse_body
 
 
@@ -269,6 +275,9 @@ class RawResponse(Response):
             self.request = response.request
             self.iter_content = response.iter_content
 
+        if not self.success():
+            self.parse_error()
+
     def success(self):
         """
         Determine if our request was successful.
@@ -281,8 +290,14 @@ class RawResponse(Response):
         """
         # pylint: disable=E1101
         import requests
-        return self.status in [requests.codes.ok, requests.codes.created,
-                               httplib.OK, httplib.CREATED, httplib.ACCEPTED]
+
+        return self.status in [
+            requests.codes.ok,
+            requests.codes.created,
+            httplib.OK,
+            httplib.CREATED,
+            httplib.ACCEPTED,
+        ]
 
     @property
     def response(self):
@@ -310,12 +325,14 @@ class Connection(object):
     """
     A Base Connection class to derive from.
     """
+
     conn_class = LibcloudConnection
 
     responseCls = Response
     rawResponseCls = RawResponse
+    retryCls = Retry
     connection = None
-    host = '127.0.0.1'  # type: str
+    host = "127.0.0.1"  # type: str
     port = 443
     timeout = None  # type: Optional[Union[int, float]]
     secure = 1
@@ -327,8 +344,17 @@ class Connection(object):
 
     allow_insecure = True
 
-    def __init__(self, secure=True, host=None, port=None, url=None,
-                 timeout=None, proxy_url=None, retry_delay=None, backoff=None):
+    def __init__(
+        self,
+        secure=True,
+        host=None,
+        port=None,
+        url=None,
+        timeout=None,
+        proxy_url=None,
+        retry_delay=None,
+        backoff=None,
+    ):
         self.secure = secure and 1 or 0
         self.ua = []
         self.context = {}
@@ -336,10 +362,11 @@ class Connection(object):
         if not self.allow_insecure and not secure:
             # TODO: We should eventually switch to whitelist instead of
             # blacklist approach
-            raise ValueError('Non https connections are not allowed (use '
-                             'secure=True)')
+            raise ValueError(
+                "Non https connections are not allowed (use " "secure=True)"
+            )
 
-        self.request_path = ''
+        self.request_path = ""
 
         if host:
             self.host = host
@@ -353,8 +380,12 @@ class Connection(object):
                 self.port = 80
 
         if url:
-            (self.host, self.port, self.secure,
-             self.request_path) = self._tuple_from_url(url)
+            (
+                self.host,
+                self.port,
+                self.secure,
+                self.request_path,
+            ) = self._tuple_from_url(url)
 
         self.timeout = timeout or self.timeout
         self.retry_delay = retry_delay
@@ -380,7 +411,7 @@ class Connection(object):
 
     def set_context(self, context):
         if not isinstance(context, dict):
-            raise TypeError('context needs to be a dictionary')
+            raise TypeError("context needs to be a dictionary")
 
         self.context = context
 
@@ -390,11 +421,10 @@ class Connection(object):
     def _tuple_from_url(self, url):
         secure = 1
         port = None
-        (scheme, netloc, request_path, param,
-         query, fragment) = urlparse.urlparse(url)
+        (scheme, netloc, request_path, param, query, fragment) = urlparse.urlparse(url)
 
-        if scheme not in ['http', 'https']:
-            raise LibcloudError('Invalid scheme: %s in url %s' % (scheme, url))
+        if scheme not in ["http", "https"]:
+            raise LibcloudError("Invalid scheme: %s in url %s" % (scheme, url))
 
         if scheme == "http":
             secure = 0
@@ -430,13 +460,12 @@ class Connection(object):
         connection = None
         secure = self.secure
 
-        if getattr(self, 'base_url', None) and base_url is None:
-            (host, port,
-             secure, request_path) = \
-                self._tuple_from_url(getattr(self, 'base_url'))
+        if getattr(self, "base_url", None) and base_url is None:
+            (host, port, secure, request_path) = self._tuple_from_url(
+                getattr(self, "base_url")
+            )
         elif base_url is not None:
-            (host, port,
-             secure, request_path) = self._tuple_from_url(base_url)
+            (host, port, secure, request_path) = self._tuple_from_url(base_url)
         else:
             host = host or self.host
             port = port or self.port
@@ -444,26 +473,26 @@ class Connection(object):
         # Make sure port is an int
         port = int(port)
 
-        if not hasattr(kwargs, 'host'):
-            kwargs.update({'host': host})
+        if not hasattr(kwargs, "host"):
+            kwargs.update({"host": host})
 
-        if not hasattr(kwargs, 'port'):
-            kwargs.update({'port': port})
+        if not hasattr(kwargs, "port"):
+            kwargs.update({"port": port})
 
-        if not hasattr(kwargs, 'secure'):
-            kwargs.update({'secure': self.secure})
+        if not hasattr(kwargs, "secure"):
+            kwargs.update({"secure": self.secure})
 
-        if not hasattr(kwargs, 'key_file') and hasattr(self, 'key_file'):
-            kwargs.update({'key_file': getattr(self, 'key_file')})
+        if not hasattr(kwargs, "key_file") and hasattr(self, "key_file"):
+            kwargs.update({"key_file": getattr(self, "key_file")})
 
-        if not hasattr(kwargs, 'cert_file') and hasattr(self, 'cert_file'):
-            kwargs.update({'cert_file': getattr(self, 'cert_file')})
+        if not hasattr(kwargs, "cert_file") and hasattr(self, "cert_file"):
+            kwargs.update({"cert_file": getattr(self, "cert_file")})
 
         if self.timeout:
-            kwargs.update({'timeout': self.timeout})
+            kwargs.update({"timeout": self.timeout})
 
         if self.proxy_url:
-            kwargs.update({'proxy_url': self.proxy_url})
+            kwargs.update({"proxy_url": self.proxy_url})
 
         connection = self.conn_class(**kwargs)
         # You can uncoment this line, if you setup a reverse proxy server
@@ -475,15 +504,16 @@ class Connection(object):
         self.connection = connection
 
     def _user_agent(self):
-        user_agent_suffix = ' '.join(['(%s)' % x for x in self.ua])
+        user_agent_suffix = " ".join(["(%s)" % x for x in self.ua])
 
         if self.driver:
-            user_agent = 'libcloud/%s (%s) %s' % (
+            user_agent = "libcloud/%s (%s) %s" % (
                 libcloud.__version__,
-                self.driver.name, user_agent_suffix)
+                self.driver.name,
+                user_agent_suffix,
+            )
         else:
-            user_agent = 'libcloud/%s %s' % (
-                libcloud.__version__, user_agent_suffix)
+            user_agent = "libcloud/%s %s" % (libcloud.__version__, user_agent_suffix)
 
         return user_agent
 
@@ -499,8 +529,18 @@ class Connection(object):
         """
         self.ua.append(token)
 
-    def request(self, action, params=None, data=None, headers=None,
-                method='GET', raw=False, stream=False, json=None):
+    def request(
+        self,
+        action,
+        params=None,
+        data=None,
+        headers=None,
+        method="GET",
+        raw=False,
+        stream=False,
+        json=None,
+        retry_failed=None,
+    ):
         """
         Request a given `action`.
 
@@ -535,6 +575,10 @@ class Connection(object):
                     and allow streaming of the response data
                     (for downloading large files)
 
+        :param retry_failed: True if failed requests should be retried. This
+                              argument can override module level constant and
+                              environment variable value on per-request basis.
+
         :return: An :class:`Response` instance.
         :rtype: :class:`Response` instance
 
@@ -549,8 +593,15 @@ class Connection(object):
         else:
             headers = copy.copy(headers)
 
-        retry_enabled = os.environ.get('LIBCLOUD_RETRY_FAILED_HTTP_REQUESTS',
-                                       False) or RETRY_FAILED_HTTP_REQUESTS
+        retry_enabled = (
+            os.environ.get("LIBCLOUD_RETRY_FAILED_HTTP_REQUESTS", False)
+            or RETRY_FAILED_HTTP_REQUESTS
+        )
+
+        # Method level argument has precedence over module level constant and
+        # environment variable
+        if retry_failed is not None:
+            retry_enabled = retry_failed
 
         action = self.morph_action_hook(action)
         self.action = action
@@ -561,24 +612,24 @@ class Connection(object):
         params = self.add_default_params(params)
 
         # Add cache busting parameters (if enabled)
-        if self.cache_busting and method == 'GET':
+        if self.cache_busting and method == "GET":
             params = self._add_cache_busting_to_params(params=params)
 
         # Extend default headers
         headers = self.add_default_headers(headers)
 
         # We always send a user-agent header
-        headers.update({'User-Agent': self._user_agent()})
+        headers.update({"User-Agent": self._user_agent()})
 
         # Indicate that we support gzip and deflate compression
-        headers.update({'Accept-Encoding': 'gzip,deflate'})
+        headers.update({"Accept-Encoding": "gzip,deflate"})
 
         port = int(self.port)
 
         if port not in (80, 443):
-            headers.update({'Host': "%s:%d" % (self.host, port)})
+            headers.update({"Host": "%s:%d" % (self.host, port)})
         else:
-            headers.update({'Host': self.host})
+            headers.update({"Host": self.host})
 
         if data:
             data = self.encode_data(data)
@@ -586,10 +637,10 @@ class Connection(object):
         params, headers = self.pre_connect_hook(params, headers)
 
         if params:
-            if '?' in action:
-                url = '&'.join((action, urlencode(params, doseq=True)))
+            if "?" in action:
+                url = "&".join((action, urlencode(params, doseq=True)))
             else:
-                url = '?'.join((action, urlencode(params, doseq=True)))
+                url = "?".join((action, urlencode(params, doseq=True)))
         else:
             url = action
 
@@ -597,9 +648,32 @@ class Connection(object):
         if self.connection is None:
             self.connect()
 
+        request_to_be_executed = self._retryable_request
+
+        if retry_enabled:
+            retry_request = self.retryCls(
+                retry_delay=self.retry_delay, timeout=self.timeout, backoff=self.backoff
+            )
+            request_to_be_executed = retry_request(self._retryable_request)
+
+        return request_to_be_executed(
+            url=url, method=method, raw=raw, stream=stream, headers=headers, data=data
+        )
+
+    def _retryable_request(
+        self,
+        url: str,
+        data: bytes,
+        headers: Dict[str, Any],
+        method: str,
+        raw: bool,
+        stream: bool,
+    ) -> Union[RawResponse, Response]:
         try:
             # @TODO: Should we just pass File object as body to request method
             # instead of dealing with splitting and sending the file ourselves?
+            assert self.connection is not None
+
             if raw:
                 self.connection.prepared_request(
                     method=method,
@@ -607,35 +681,29 @@ class Connection(object):
                     body=data,
                     headers=headers,
                     raw=raw,
-                    stream=stream)
+                    stream=stream,
+                )
             else:
-                if retry_enabled:
-                    retry_request = retry(timeout=self.timeout,
-                                          retry_delay=self.retry_delay,
-                                          backoff=self.backoff)
-                    retry_request(self.connection.request)(method=method,
-                                                           url=url,
-                                                           body=data,
-                                                           headers=headers,
-                                                           stream=stream)
-                else:
-                    self.connection.request(method=method, url=url, body=data,
-                                            headers=headers, stream=stream)
+                self.connection.request(
+                    method=method, url=url, body=data, headers=headers, stream=stream
+                )
+
         except socket.gaierror as e:
             message = str(e)
-            errno = getattr(e, 'errno', None)
+            errno = getattr(e, "errno", None)
 
             if errno == -5:
                 # Throw a more-friendly exception on "no address associated
-                # with hostname" error. This error could simpli indicate that
+                # with hostname" error. This error could simply indicate that
                 # "host" Connection class attribute is set to an incorrect
                 # value
                 class_name = self.__class__.__name__
-                msg = ('%s. Perhaps "host" Connection class attribute '
-                       '(%s.connection) is set to an invalid, non-hostname '
-                       'value (%s)?' %
-                       (message, class_name, self.host))
-                raise socket.gaierror(msg)
+                msg = (
+                    '%s. Perhaps "host" Connection class attribute '
+                    "(%s.connection) is set to an invalid, non-hostname "
+                    "value (%s)?" % (message, class_name, self.host)
+                )
+                raise socket.gaierror(msg)  # type: ignore
             self.reset_context()
             raise e
         except ssl.SSLError as e:
@@ -644,12 +712,10 @@ class Connection(object):
 
         if raw:
             responseCls = self.rawResponseCls
-            kwargs = {'connection': self,
-                      'response': self.connection.getresponse()}
+            kwargs = {"connection": self, "response": self.connection.getresponse()}
         else:
             responseCls = self.responseCls
-            kwargs = {'connection': self,
-                      'response': self.connection.getresponse()}
+            kwargs = {"connection": self, "response": self.connection.getresponse()}
 
         try:
             response = responseCls(**kwargs)
@@ -674,11 +740,12 @@ class Connection(object):
             # valid - e.g. for S3 paths - /bucket//path1/path2.txt
             return self.request_path + action
 
-        url = urlparse.urljoin(self.request_path.lstrip('/').rstrip('/') +
-                               '/', action.lstrip('/'))
+        url = urlparse.urljoin(
+            self.request_path.lstrip("/").rstrip("/") + "/", action.lstrip("/")
+        )
 
-        if not url.startswith('/'):
-            return '/' + url
+        if not url.startswith("/"):
+            return "/" + url
         else:
             return url
 
@@ -732,12 +799,12 @@ class Connection(object):
         Note: This should only be used with *naughty* providers which use
         excessive caching of responses.
         """
-        cache_busting_value = binascii.hexlify(os.urandom(8)).decode('ascii')
+        cache_busting_value = binascii.hexlify(os.urandom(8)).decode("ascii")
 
         if isinstance(params, dict):
-            params['cache-busting'] = cache_busting_value
+            params["cache-busting"] = cache_busting_value
         else:
-            params.append(('cache-busting', cache_busting_value))
+            params.append(("cache-busting", cache_busting_value))
 
         return params
 
@@ -750,12 +817,14 @@ class PollingConnection(Connection):
     waits until the job has finished.
     If job doesn't finish in timeout seconds, an Exception thrown.
     """
+
     poll_interval = 0.5
     timeout = 200
-    request_method = 'request'
+    request_method = "request"
 
-    def async_request(self, action, params=None, data=None, headers=None,
-                      method='GET', context=None):
+    def async_request(
+        self, action, params=None, data=None, headers=None, method="GET", context=None
+    ):
         """
         Perform an 'async' request to the specified path. Keep in mind that
         this function is *blocking* and 'async' in this case means that the
@@ -798,14 +867,18 @@ class PollingConnection(Connection):
         """
 
         request = getattr(self, self.request_method)
-        kwargs = self.get_request_kwargs(action=action, params=params,
-                                         data=data, headers=headers,
-                                         method=method,
-                                         context=context)
+        kwargs = self.get_request_kwargs(
+            action=action,
+            params=params,
+            data=data,
+            headers=headers,
+            method=method,
+            context=context,
+        )
         response = request(**kwargs)
-        kwargs = self.get_poll_request_kwargs(response=response,
-                                              context=context,
-                                              request_kwargs=kwargs)
+        kwargs = self.get_poll_request_kwargs(
+            response=response, context=context, request_kwargs=kwargs
+        )
 
         end = time.time() + self.timeout
         completed = False
@@ -816,19 +889,24 @@ class PollingConnection(Connection):
                 time.sleep(self.poll_interval)
 
         if not completed:
-            raise LibcloudError('Job did not complete in %s seconds' %
-                                (self.timeout))
+            raise LibcloudError("Job did not complete in %s seconds" % (self.timeout))
 
         return response
 
-    def get_request_kwargs(self, action, params=None, data=None, headers=None,
-                           method='GET', context=None):
+    def get_request_kwargs(
+        self, action, params=None, data=None, headers=None, method="GET", context=None
+    ):
         """
         Arguments which are passed to the initial request() call inside
         async_request.
         """
-        kwargs = {'action': action, 'params': params, 'data': data,
-                  'headers': headers, 'method': method}
+        kwargs = {
+            "action": action,
+            "params": params,
+            "data": data,
+            "headers": headers,
+            "method": method,
+        }
         return kwargs
 
     def get_poll_request_kwargs(self, response, context, request_kwargs):
@@ -845,7 +923,7 @@ class PollingConnection(Connection):
 
         :return ``dict`` Keyword arguments
         """
-        raise NotImplementedError('get_poll_request_kwargs not implemented')
+        raise NotImplementedError("get_poll_request_kwargs not implemented")
 
     def has_completed(self, response):
         """
@@ -856,25 +934,40 @@ class PollingConnection(Connection):
 
         :return ``bool`` True if the job has completed, False otherwise.
         """
-        raise NotImplementedError('has_completed not implemented')
+        raise NotImplementedError("has_completed not implemented")
 
 
 class ConnectionKey(Connection):
     """
     Base connection class which accepts a single ``key`` argument.
     """
-    def __init__(self, key, secure=True, host=None, port=None, url=None,
-                 timeout=None, proxy_url=None, backoff=None, retry_delay=None):
+
+    def __init__(
+        self,
+        key,
+        secure=True,
+        host=None,
+        port=None,
+        url=None,
+        timeout=None,
+        proxy_url=None,
+        backoff=None,
+        retry_delay=None,
+    ):
         """
         Initialize `user_id` and `key`; set `secure` to an ``int`` based on
         passed value.
         """
-        super(ConnectionKey, self).__init__(secure=secure, host=host,
-                                            port=port, url=url,
-                                            timeout=timeout,
-                                            proxy_url=proxy_url,
-                                            backoff=backoff,
-                                            retry_delay=retry_delay)
+        super(ConnectionKey, self).__init__(
+            secure=secure,
+            host=host,
+            port=port,
+            url=url,
+            timeout=timeout,
+            proxy_url=proxy_url,
+            backoff=backoff,
+            retry_delay=retry_delay,
+        )
         self.key = key
 
 
@@ -882,18 +975,33 @@ class CertificateConnection(Connection):
     """
     Base connection class which accepts a single ``cert_file`` argument.
     """
-    def __init__(self, cert_file, secure=True, host=None, port=None, url=None,
-                 proxy_url=None, timeout=None, backoff=None, retry_delay=None):
+
+    def __init__(
+        self,
+        cert_file,
+        secure=True,
+        host=None,
+        port=None,
+        url=None,
+        proxy_url=None,
+        timeout=None,
+        backoff=None,
+        retry_delay=None,
+    ):
         """
         Initialize `cert_file`; set `secure` to an ``int`` based on
         passed value.
         """
-        super(CertificateConnection, self).__init__(secure=secure, host=host,
-                                                    port=port, url=url,
-                                                    timeout=timeout,
-                                                    backoff=backoff,
-                                                    retry_delay=retry_delay,
-                                                    proxy_url=proxy_url)
+        super(CertificateConnection, self).__init__(
+            secure=secure,
+            host=host,
+            port=port,
+            url=url,
+            timeout=timeout,
+            backoff=backoff,
+            retry_delay=retry_delay,
+            proxy_url=proxy_url,
+        )
 
         self.cert_file = cert_file
 
@@ -906,21 +1014,35 @@ class KeyCertificateConnection(CertificateConnection):
 
     key_file = None
 
-    def __init__(self, key_file, cert_file, secure=True, host=None, port=None,
-                 url=None, proxy_url=None, timeout=None, backoff=None,
-                 retry_delay=None, ca_cert=None):
+    def __init__(
+        self,
+        key_file,
+        cert_file,
+        secure=True,
+        host=None,
+        port=None,
+        url=None,
+        proxy_url=None,
+        timeout=None,
+        backoff=None,
+        retry_delay=None,
+        ca_cert=None,
+    ):
         """
         Initialize `cert_file`; set `secure` to an ``int`` based on
         passed value.
         """
-        super(KeyCertificateConnection, self).__init__(cert_file,
-                                                       secure=secure,
-                                                       host=host,
-                                                       port=port, url=url,
-                                                       timeout=timeout,
-                                                       backoff=backoff,
-                                                       retry_delay=retry_delay,
-                                                       proxy_url=proxy_url)
+        super(KeyCertificateConnection, self).__init__(
+            cert_file,
+            secure=secure,
+            host=host,
+            port=port,
+            url=url,
+            timeout=timeout,
+            backoff=backoff,
+            retry_delay=retry_delay,
+            proxy_url=proxy_url,
+        )
 
         self.key_file = key_file
 
@@ -932,15 +1054,30 @@ class ConnectionUserAndKey(ConnectionKey):
 
     user_id = None  # type: int
 
-    def __init__(self, user_id, key, secure=True, host=None, port=None,
-                 url=None, timeout=None, proxy_url=None,
-                 backoff=None, retry_delay=None):
-        super(ConnectionUserAndKey, self).__init__(key, secure=secure,
-                                                   host=host, port=port,
-                                                   url=url, timeout=timeout,
-                                                   backoff=backoff,
-                                                   retry_delay=retry_delay,
-                                                   proxy_url=proxy_url)
+    def __init__(
+        self,
+        user_id,
+        key,
+        secure=True,
+        host=None,
+        port=None,
+        url=None,
+        timeout=None,
+        proxy_url=None,
+        backoff=None,
+        retry_delay=None,
+    ):
+        super(ConnectionUserAndKey, self).__init__(
+            key,
+            secure=secure,
+            host=host,
+            port=port,
+            url=url,
+            timeout=timeout,
+            backoff=backoff,
+            retry_delay=retry_delay,
+            proxy_url=proxy_url,
+        )
         self.user_id = user_id
 
 
@@ -951,8 +1088,17 @@ class BaseDriver(object):
 
     connectionCls = ConnectionKey  # type: Type[Connection]
 
-    def __init__(self, key, secret=None, secure=True, host=None, port=None,
-                 api_version=None, region=None, **kwargs):
+    def __init__(
+        self,
+        key,
+        secret=None,
+        secure=True,
+        host=None,
+        port=None,
+        api_version=None,
+        region=None,
+        **kwargs,
+    ):
         """
         :param    key:    API key or username to be used (required)
         :type     key:    ``str``
@@ -984,6 +1130,19 @@ class BaseDriver(object):
         self.key = key
         self.secret = secret
         self.secure = secure
+        self.api_version = api_version
+        self.region = region
+
+        conn_kwargs = self._ex_connection_class_kwargs()
+        conn_kwargs.update(
+            {
+                "timeout": kwargs.pop("timeout", None),
+                "retry_delay": kwargs.pop("retry_delay", None),
+                "backoff": kwargs.pop("backoff", None),
+                "proxy_url": kwargs.pop("proxy_url", None),
+            }
+        )
+
         args = [self.key]
 
         if self.secret is not None:
@@ -991,22 +1150,15 @@ class BaseDriver(object):
 
         args.append(secure)
 
+        host = conn_kwargs.pop("host", None) or host
+
         if host is not None:
             args.append(host)
 
         if port is not None:
             args.append(port)
 
-        self.api_version = api_version
-        self.region = region
-
-        conn_kwargs = self._ex_connection_class_kwargs()
-        conn_kwargs.update({'timeout': kwargs.pop('timeout', None),
-                            'retry_delay': kwargs.pop('retry_delay', None),
-                            'backoff': kwargs.pop('backoff', None),
-                            'proxy_url': kwargs.pop('proxy_url', None)})
         self.connection = self.connectionCls(*args, **conn_kwargs)
-
         self.connection.driver = self
         self.connection.connect()
 
