@@ -31,7 +31,7 @@ class BrightboxResponse(JsonResponse):
         return httplib.OK <= self.status < httplib.BAD_REQUEST
 
     def parse_body(self):
-        if self.headers['content-type'].split(';')[0] == 'application/json':
+        if self.headers["content-type"].split(";")[0] == "application/json":
             return super(BrightboxResponse, self).parse_body()
         else:
             return self.body
@@ -39,13 +39,13 @@ class BrightboxResponse(JsonResponse):
     def parse_error(self):
         response = super(BrightboxResponse, self).parse_body()
 
-        if 'error' in response:
-            if response['error'] in ['invalid_client', 'unauthorized_client']:
-                raise InvalidCredsError(response['error'])
+        if "error" in response:
+            if response["error"] in ["invalid_client", "unauthorized_client"]:
+                raise InvalidCredsError(response["error"])
 
-            return response['error']
-        elif 'error_name' in response:
-            return '%s: %s' % (response['error_name'], response['errors'][0])
+            return response["error"]
+        elif "error_name" in response:
+            return "%s: %s" % (response["error_name"], response["errors"][0])
 
         return self.body
 
@@ -55,44 +55,48 @@ class BrightboxConnection(ConnectionUserAndKey):
     Connection class for the Brightbox driver
     """
 
-    host = 'api.gb1.brightbox.com'
+    host = "api.gb1.brightbox.com"
     responseCls = BrightboxResponse
 
     def _fetch_oauth_token(self):
-        body = json.dumps({'client_id': self.user_id, 'grant_type': 'none'})
+        body = json.dumps({"client_id": self.user_id, "grant_type": "none"})
 
-        authorization = 'Basic ' + str(base64_encode_string(b('%s:%s' %
-                                       (self.user_id, self.key)))).rstrip()
+        authorization = (
+            "Basic "
+            + str(base64_encode_string(b("%s:%s" % (self.user_id, self.key)))).rstrip()
+        )
 
         self.connect()
 
         headers = {
-            'Host': self.host,
-            'User-Agent': self._user_agent(),
-            'Authorization': authorization,
-            'Content-Type': 'application/json',
-            'Content-Length': str(len(body))
+            "Host": self.host,
+            "User-Agent": self._user_agent(),
+            "Authorization": authorization,
+            "Content-Type": "application/json",
+            "Content-Length": str(len(body)),
         }
 
         # pylint: disable=assignment-from-no-return
-        response = self.connection.request(method='POST', url='/token',
-                                           body=body, headers=headers)
+        response = self.connection.request(
+            method="POST", url="/token", body=body, headers=headers
+        )
 
         if response.status == httplib.OK:
-            return json.loads(response.read())['access_token']
+            return json.loads(response.read())["access_token"]
         else:
             responseCls = BrightboxResponse(
-                response=response.getresponse(), connection=self)
+                response=response.getresponse(), connection=self
+            )
             message = responseCls.parse_error()
             raise InvalidCredsError(message)
 
     def add_default_headers(self, headers):
         try:
-            headers['Authorization'] = 'OAuth ' + self.token
+            headers["Authorization"] = "OAuth " + self.token
         except AttributeError:
             self.token = self._fetch_oauth_token()
 
-            headers['Authorization'] = 'OAuth ' + self.token
+            headers["Authorization"] = "OAuth " + self.token
 
         return headers
 
