@@ -26,6 +26,7 @@ from typing import IO
 from typing import cast
 
 import os
+import re
 import binascii
 
 from libcloud.utils.py3 import basestring, PY3
@@ -51,16 +52,18 @@ class Deployment(object):
 
         :return: :class:`Node`
         """
-        raise NotImplementedError(
-            'run not implemented for this deployment')
+        raise NotImplementedError("run not implemented for this deployment")
 
     def _get_string_value(self, argument_name, argument_value):
-        if not isinstance(argument_value, basestring) and \
-           not hasattr(argument_value, 'read'):
-            raise TypeError('%s argument must be a string or a file-like '
-                            'object' % (argument_name))
+        if not isinstance(argument_value, basestring) and not hasattr(
+            argument_value, "read"
+        ):
+            raise TypeError(
+                "%s argument must be a string or a file-like "
+                "object" % (argument_name)
+            )
 
-        if hasattr(argument_value, 'read'):
+        if hasattr(argument_value, "read"):
             argument_value = argument_value.read()
 
         return argument_value
@@ -78,8 +81,7 @@ class SSHKeyDeployment(Deployment):
         :keyword key: Contents of the public key write or a file object which
                       can be read.
         """
-        self.key = self._get_string_value(argument_name='key',
-                                          argument_value=key)
+        self.key = self._get_string_value(argument_name="key", argument_value=key)
 
     def run(self, node, client):
         # type: (Node, BaseSSHClient) -> Node
@@ -88,9 +90,16 @@ class SSHKeyDeployment(Deployment):
 
         See also :class:`Deployment.run`
         """
-        client.put(".ssh/authorized_keys", contents=self.key, mode='a')
+        client.put(".ssh/authorized_keys", contents=self.key, mode="a")
         return node
 
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        key = self.key[:100]
+        return "<SSHKeyDeployment key=%s...>" % (key)
+
 
 class FileDeployment(Deployment):
     """
@@ -118,13 +127,16 @@ class FileDeployment(Deployment):
         """
         perms = int(oct(os.stat(self.source).st_mode)[4:], 8)
 
-        with open(self.source, 'rb') as fp:
-            content = fp.read()
-
-        client.put(path=self.target, chmod=perms,
-                   contents=content)
+        with open(self.source, "rb") as fp:
+            client.putfo(path=self.target, chmod=perms, fo=fp)
         return node
 
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        return "<FileDeployment source=%s, target=%s>" % (self.source, self.target)
+
 
 class ScriptDeployment(Deployment):
     """
@@ -138,13 +150,14 @@ class ScriptDeployment(Deployment):
     you are running a plan shell script.
     """
 
-    def __init__(self,
-                 script,  # type: str
-                 args=None,  # type: Optional[List[str]]
-                 name=None,  # type: Optional[str]
-                 delete=False,  # type bool
-                 timeout=None  # type: Optional[float]
-                 ):
+    def __init__(
+        self,
+        script,  # type: str
+        args=None,  # type: Optional[List[str]]
+        name=None,  # type: Optional[str]
+        delete=False,  # type bool
+        timeout=None,  # type: Optional[float]
+    ):
         # type: (...) -> None
         """
         :type script: ``str``
@@ -164,8 +177,7 @@ class ScriptDeployment(Deployment):
         :param timeout: Optional run timeout for this command.
         :type timeout: ``float``
         """
-        script = self._get_string_value(argument_name='script',
-                                        argument_value=script)
+        script = self._get_string_value(argument_name="script", argument_value=script)
 
         self.script = script
         self.args = args or []
@@ -179,11 +191,11 @@ class ScriptDeployment(Deployment):
         if self.name is None:
             # File is put under user's home directory
             # (~/libcloud_deployment_<random_string>.sh)
-            random_string = ''  # type: Union[str, bytes]
+            random_string = ""  # type: Union[str, bytes]
             random_string = binascii.hexlify(os.urandom(4))
             random_string = cast(bytes, random_string)
-            random_string = random_string.decode('ascii')
-            self.name = 'libcloud_deployment_%s.sh' % (random_string)
+            random_string = random_string.decode("ascii")
+            self.name = "libcloud_deployment_%s.sh" % (random_string)
 
     def run(self, node, client):
         # type: (Node, BaseSSHClient) -> Node
@@ -193,13 +205,18 @@ class ScriptDeployment(Deployment):
         See also :class:`Deployment.run`
         """
         self.name = cast(str, self.name)
-        file_path = client.put(path=self.name, chmod=int('755', 8),
-                               contents=self.script)
-
+        file_path = client.put(
+            path=self.name, chmod=int("755", 8), contents=self.script
+        )
         # Pre-pend cwd if user specified a relative path
-        if self.name and self.name[0] != '/':
+        if self.name and (
+            self.name[0] not in ["/", "\\"] and not re.match(r"^\w\:.*$", file_path)
+        ):
             base_path = os.path.dirname(file_path)
             name = os.path.join(base_path, self.name)
+        elif self.name and (self.name[0] == "\\" or re.match(r"^\w\:.*$", file_path)):
+            # Absolute Windows path
+            name = file_path
         else:
             self.name = cast(str, self.name)
             name = self.name
@@ -208,12 +225,13 @@ class ScriptDeployment(Deployment):
 
         if self.args:
             # Append arguments to the command
-            cmd = '%s %s' % (name, ' '.join(self.args))
+            cmd = "%s %s" % (name, " ".join(self.args))
         else:
             cmd = name
 
-        self.stdout, self.stderr, self.exit_status = \
-            client.run(cmd, timeout=self.timeout)
+        self.stdout, self.stderr, self.exit_status = client.run(
+            cmd, timeout=self.timeout
+        )
 
         if self.delete:
             client.delete(self.name)
@@ -224,18 +242,20 @@ class ScriptDeployment(Deployment):
         return self.__repr__()
 
     def __repr__(self):
-        script = self.script[:15] + '...'
+        script = self.script[:15] + "..."
         exit_status = self.exit_status
 
         if exit_status is not None:
-            stdout = self.stdout[:30] + '...'
-            stderr = self.stderr[:30] + '...'
+            stdout = self.stdout[:30] + "..."
+            stderr = self.stderr[:30] + "..."
         else:
-            exit_status = 'script didn\'t run yet'
+            exit_status = "script didn't run yet"
             stdout = None
             stderr = None
-        return ("<ScriptDeployment script=%s, exit_status=%s, stdout=%s, "
-                "stderr=%s>" % (script, exit_status, stdout, stderr))
+        return (
+            "<ScriptDeployment script=%s, exit_status=%s, stdout=%s, "
+            "stderr=%s>" % (script, exit_status, stdout, stderr)
+        )
 
 
 class ScriptFileDeployment(ScriptDeployment):
@@ -245,13 +265,14 @@ class ScriptFileDeployment(ScriptDeployment):
     the script content.
     """
 
-    def __init__(self,
-                 script_file,  # type: str
-                 args=None,  # type: Optional[List[str]]
-                 name=None,  # type: Optional[str]
-                 delete=False,  # type bool
-                 timeout=None  # type: Optional[float]
-                 ):
+    def __init__(
+        self,
+        script_file,  # type: str
+        args=None,  # type: Optional[List[str]]
+        name=None,  # type: Optional[str]
+        delete=False,  # type bool
+        timeout=None,  # type: Optional[float]
+    ):
         # type: (...) -> None
         """
         :type script_file: ``str``
@@ -272,24 +293,23 @@ class ScriptFileDeployment(ScriptDeployment):
         :param timeout: Optional run timeout for this command.
         :type timeout: ``float``
         """
-        with open(script_file, 'rb') as fp:
+        with open(script_file, "rb") as fp:
             content = fp.read()  # type: Union[bytes, str]
 
         if PY3:
             content = cast(bytes, content)
-            content = content.decode('utf-8')
+            content = content.decode("utf-8")
 
-        super(ScriptFileDeployment, self).__init__(script=content,
-                                                   args=args,
-                                                   name=name,
-                                                   delete=delete,
-                                                   timeout=timeout)
+        super(ScriptFileDeployment, self).__init__(
+            script=content, args=args, name=name, delete=delete, timeout=timeout
+        )
 
 
 class MultiStepDeployment(Deployment):
     """
     Runs a chain of Deployment steps.
     """
+
     def __init__(self, add=None):
         # type: (Optional[Union[Deployment, List[Deployment]]]) -> None
         """
@@ -325,3 +345,16 @@ class MultiStepDeployment(Deployment):
         for s in self.steps:
             node = s.run(node, client)
         return node
+
+    def __str__(self):
+        return self.__repr__()
+
+    def __repr__(self):
+        steps = []
+
+        for step in self.steps:
+            steps.append(str(step))
+
+        steps = ", ".join(steps)
+
+        return "<MultiStepDeployment steps=[%s]>" % (steps)
