@@ -17,8 +17,13 @@ Gandi driver for compute
 """
 from datetime import datetime
 
-from libcloud.common.gandi import BaseGandiDriver, GandiException,\
-    NetworkInterface, IPAddress, Disk
+from libcloud.common.gandi import (
+    BaseGandiDriver,
+    GandiException,
+    NetworkInterface,
+    IPAddress,
+    Disk,
+)
 from libcloud.compute.base import KeyPair
 from libcloud.compute.base import StorageVolume
 from libcloud.compute.types import NodeState, Provider
@@ -27,50 +32,50 @@ from libcloud.compute.base import NodeSize, NodeImage, NodeLocation
 
 
 NODE_STATE_MAP = {
-    'running': NodeState.RUNNING,
-    'halted': NodeState.TERMINATED,
-    'paused': NodeState.TERMINATED,
-    'locked': NodeState.TERMINATED,
-    'being_created': NodeState.PENDING,
-    'invalid': NodeState.UNKNOWN,
-    'legally_locked': NodeState.PENDING,
-    'deleted': NodeState.TERMINATED
+    "running": NodeState.RUNNING,
+    "halted": NodeState.TERMINATED,
+    "paused": NodeState.TERMINATED,
+    "locked": NodeState.TERMINATED,
+    "being_created": NodeState.PENDING,
+    "invalid": NodeState.UNKNOWN,
+    "legally_locked": NodeState.PENDING,
+    "deleted": NodeState.TERMINATED,
 }
 
 NODE_PRICE_HOURLY_USD = 0.02
 
 INSTANCE_TYPES = {
-    'small': {
-        'id': 'small',
-        'name': 'Small instance',
-        'cpu': 1,
-        'memory': 256,
-        'disk': 3,
-        'bandwidth': 10240,
+    "small": {
+        "id": "small",
+        "name": "Small instance",
+        "cpu": 1,
+        "memory": 256,
+        "disk": 3,
+        "bandwidth": 10240,
     },
-    'medium': {
-        'id': 'medium',
-        'name': 'Medium instance',
-        'cpu': 1,
-        'memory': 1024,
-        'disk': 20,
-        'bandwidth': 10240,
+    "medium": {
+        "id": "medium",
+        "name": "Medium instance",
+        "cpu": 1,
+        "memory": 1024,
+        "disk": 20,
+        "bandwidth": 10240,
     },
-    'large': {
-        'id': 'large',
-        'name': 'Large instance',
-        'cpu': 2,
-        'memory': 2048,
-        'disk': 50,
-        'bandwidth': 10240,
+    "large": {
+        "id": "large",
+        "name": "Large instance",
+        "cpu": 2,
+        "memory": 2048,
+        "disk": 50,
+        "bandwidth": 10240,
     },
-    'x-large': {
-        'id': 'x-large',
-        'name': 'Extra Large instance',
-        'cpu': 4,
-        'memory': 4096,
-        'disk': 100,
-        'bandwidth': 10240,
+    "x-large": {
+        "id": "x-large",
+        "name": "Extra Large instance",
+        "cpu": 4,
+        "memory": 4096,
+        "disk": 100,
+        "bandwidth": 10240,
     },
 }
 
@@ -80,10 +85,11 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
     Gandi node driver
 
     """
-    api_name = 'gandi'
-    friendly_name = 'Gandi.net'
-    website = 'http://www.gandi.net/'
-    country = 'FR'
+
+    api_name = "gandi"
+    friendly_name = "Gandi.net"
+    website = "http://www.gandi.net/"
+    country = "FR"
     type = Provider.GANDI
     # TODO : which features to enable ?
     features = {}
@@ -96,47 +102,45 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
     def _resource_info(self, type, id):
         try:
-            obj = self.connection.request('hosting.%s.info' % type, int(id))
+            obj = self.connection.request("hosting.%s.info" % type, int(id))
             return obj.object
         except Exception as e:
             raise GandiException(1003, e)
 
     def _node_info(self, id):
-        return self._resource_info('vm', id)
+        return self._resource_info("vm", id)
 
     def _volume_info(self, id):
-        return self._resource_info('disk', id)
+        return self._resource_info("disk", id)
 
     # Generic methods for driver
     def _to_node(self, vm):
         return Node(
-            id=vm['id'],
-            name=vm['hostname'],
-            state=NODE_STATE_MAP.get(
-                vm['state'],
-                NodeState.UNKNOWN
-            ),
-            public_ips=vm.get('ips', []),
+            id=vm["id"],
+            name=vm["hostname"],
+            state=NODE_STATE_MAP.get(vm["state"], NodeState.UNKNOWN),
+            public_ips=vm.get("ips", []),
             private_ips=[],
             driver=self,
             extra={
-                'ai_active': vm.get('ai_active'),
-                'datacenter_id': vm.get('datacenter_id'),
-                'description': vm.get('description')
-            }
+                "ai_active": vm.get("ai_active"),
+                "datacenter_id": vm.get("datacenter_id"),
+                "description": vm.get("description"),
+            },
         )
 
     def _to_nodes(self, vms):
         return [self._to_node(v) for v in vms]
 
     def _to_volume(self, disk):
-        extra = {'can_snapshot': disk['can_snapshot']}
+        extra = {"can_snapshot": disk["can_snapshot"]}
         return StorageVolume(
-            id=disk['id'],
-            name=disk['name'],
-            size=int(disk['size']),
+            id=disk["id"],
+            name=disk["name"],
+            size=int(disk["size"]),
             driver=self,
-            extra=extra)
+            extra=extra,
+        )
 
     def _to_volumes(self, disks):
         return [self._to_volume(d) for d in disks]
@@ -148,15 +152,15 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:  List of Node objects
         :rtype:   ``list`` of :class:`Node`
         """
-        vms = self.connection.request('hosting.vm.list').object
-        ips = self.connection.request('hosting.ip.list').object
+        vms = self.connection.request("hosting.vm.list").object
+        ips = self.connection.request("hosting.ip.list").object
         for vm in vms:
-            vm['ips'] = []
+            vm["ips"] = []
             for ip in ips:
-                if vm['ifaces_id'][0] == ip['iface_id']:
-                    ip = ip.get('ip', None)
+                if vm["ifaces_id"][0] == ip["iface_id"]:
+                    ip = ip.get("ip", None)
                     if ip:
-                        vm['ips'].append(ip)
+                        vm["ips"].append(ip)
 
         nodes = self._to_nodes(vms)
         return nodes
@@ -171,14 +175,14 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:  A Node object for the node
         :rtype:   :class:`Node`
         """
-        vm = self.connection.request('hosting.vm.info', int(node_id)).object
-        ips = self.connection.request('hosting.ip.list').object
-        vm['ips'] = []
+        vm = self.connection.request("hosting.vm.info", int(node_id)).object
+        ips = self.connection.request("hosting.ip.list").object
+        vm["ips"] = []
         for ip in ips:
-            if vm['ifaces_id'][0] == ip['iface_id']:
-                ip = ip.get('ip', None)
+            if vm["ifaces_id"][0] == ip["iface_id"]:
+                ip = ip.get("ip", None)
                 if ip:
-                    vm['ips'].append(ip)
+                    vm["ips"].append(ip)
         node = self._to_node(vm)
         return node
 
@@ -192,10 +196,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:  True if successful, False if not
         :rtype:   ``bool``
         """
-        op = self.connection.request('hosting.vm.reboot', int(node.id))
-        self._wait_operation(op.object['id'])
+        op = self.connection.request("hosting.vm.reboot", int(node.id))
+        self._wait_operation(op.object["id"])
         vm = self._node_info(int(node.id))
-        if vm['state'] == 'running':
+        if vm["state"] == "running":
             return True
         return False
 
@@ -210,14 +214,14 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :rtype:   ``bool``
         """
         vm = self._node_info(node.id)
-        if vm['state'] == 'running':
+        if vm["state"] == "running":
             # Send vm_stop and wait for accomplish
-            op_stop = self.connection.request('hosting.vm.stop', int(node.id))
-            if not self._wait_operation(op_stop.object['id']):
-                raise GandiException(1010, 'vm.stop failed')
+            op_stop = self.connection.request("hosting.vm.stop", int(node.id))
+            if not self._wait_operation(op_stop.object["id"]):
+                raise GandiException(1010, "vm.stop failed")
             # Delete
-        op = self.connection.request('hosting.vm.delete', int(node.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request("hosting.vm.delete", int(node.id))
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -227,10 +231,19 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``bool``
         """
-        raise NotImplementedError(
-            'deploy_node not implemented for gandi driver')
+        raise NotImplementedError("deploy_node not implemented for gandi driver")
 
-    def create_node(self, **kwargs):
+    def create_node(
+        self,
+        name,
+        size,
+        image,
+        location=None,
+        login=None,
+        password=None,
+        inet_family=4,
+        keypairs=None,
+    ):
         """
         Create a new Gandi node
 
@@ -262,83 +275,72 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: :class:`Node`
         """
+        keypairs = keypairs or []
 
-        if not kwargs.get('login') and not kwargs.get('keypairs'):
-            raise GandiException(1020, "Login and password or ssh keypair "
-                                 "must be defined for node creation")
+        if not login and not keypairs:
+            raise GandiException(
+                1020,
+                "Login and password or ssh keypair "
+                "must be defined for node creation",
+            )
 
-        location = kwargs.get('location')
         if location and isinstance(location, NodeLocation):
             dc_id = int(location.id)
         else:
-            raise GandiException(
-                1021, 'location must be a subclass of NodeLocation')
+            raise GandiException(1021, "location must be a subclass of NodeLocation")
 
-        size = kwargs.get('size')
         if not size and not isinstance(size, NodeSize):
-            raise GandiException(
-                1022, 'size must be a subclass of NodeSize')
+            raise GandiException(1022, "size must be a subclass of NodeSize")
 
-        keypairs = kwargs.get('keypairs', [])
-        keypair_ids = [
-            k if isinstance(k, int) else k.extra['id']
-            for k in keypairs
-        ]
+        keypair_ids = [k if isinstance(k, int) else k.extra["id"] for k in keypairs]
 
         # If size name is in INSTANCE_TYPE we use new rating model
         instance = INSTANCE_TYPES.get(size.id)
-        cores = instance['cpu'] if instance else int(size.id)
+        cores = instance["cpu"] if instance else int(size.id)
 
-        src_disk_id = int(kwargs['image'].id)
+        src_disk_id = int(image.id)
 
-        disk_spec = {
-            'datacenter_id': dc_id,
-            'name': 'disk_%s' % kwargs['name']
-        }
+        disk_spec = {"datacenter_id": dc_id, "name": "disk_%s" % name}
 
         vm_spec = {
-            'datacenter_id': dc_id,
-            'hostname': kwargs['name'],
-            'memory': int(size.ram),
-            'cores': cores,
-            'bandwidth': int(size.bandwidth),
-            'ip_version': kwargs.get('inet_family', 4),
+            "datacenter_id": dc_id,
+            "hostname": name,
+            "memory": int(size.ram),
+            "cores": cores,
+            "bandwidth": int(size.bandwidth),
+            "ip_version": inet_family,
         }
 
-        if kwargs.get('login') and kwargs.get('password'):
-            vm_spec.update({
-                'login': kwargs['login'],
-                'password': kwargs['password'],  # TODO : use NodeAuthPassword
-            })
+        if login and password:
+            vm_spec.update(
+                {"login": login, "password": password}  # TODO : use NodeAuthPassword
+            )
         if keypair_ids:
-            vm_spec['keys'] = keypair_ids
+            vm_spec["keys"] = keypair_ids
 
         # Call create_from helper api. Return 3 operations : disk_create,
         # iface_create,vm_create
         (op_disk, op_iface, op_vm) = self.connection.request(
-            'hosting.vm.create_from',
-            vm_spec, disk_spec, src_disk_id
+            "hosting.vm.create_from", vm_spec, disk_spec, src_disk_id
         ).object
 
         # We wait for vm_create to finish
-        if self._wait_operation(op_vm['id']):
+        if self._wait_operation(op_vm["id"]):
             # after successful operation, get ip information
             # thru first interface
-            node = self._node_info(op_vm['vm_id'])
-            ifaces = node.get('ifaces')
+            node = self._node_info(op_vm["vm_id"])
+            ifaces = node.get("ifaces")
             if len(ifaces) > 0:
-                ips = ifaces[0].get('ips')
+                ips = ifaces[0].get("ips")
                 if len(ips) > 0:
-                    node['ip'] = ips[0]['ip']
+                    node["ip"] = ips[0]["ip"]
             return self._to_node(node)
 
         return None
 
     def _to_image(self, img):
         return NodeImage(
-            id=img['disk_id'],
-            name=img['label'],
-            driver=self.connection.driver
+            id=img["disk_id"], name=img["label"], driver=self.connection.driver
         )
 
     def list_images(self, location=None):
@@ -353,10 +355,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         """
         try:
             if location:
-                filtering = {'datacenter_id': int(location.id)}
+                filtering = {"datacenter_id": int(location.id)}
             else:
                 filtering = {}
-            images = self.connection.request('hosting.image.list', filtering)
+            images = self.connection.request("hosting.image.list", filtering)
             return [self._to_image(i) for i in images.object]
         except Exception as e:
             raise GandiException(1011, e)
@@ -364,28 +366,30 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
     def _to_size(self, id, size):
         return NodeSize(
             id=id,
-            name='%s cores' % id,
-            ram=size['memory'],
-            disk=size['disk'],
-            bandwidth=size['bandwidth'],
-            price=(self._get_size_price(size_id='1') * id),
+            name="%s cores" % id,
+            ram=size["memory"],
+            disk=size["disk"],
+            bandwidth=size["bandwidth"],
+            price=(self._get_size_price(size_id="1") * id),
             driver=self.connection.driver,
         )
 
     def _instance_type_to_size(self, instance):
         return NodeSize(
-            id=instance['id'],
-            name=instance['name'],
-            ram=instance['memory'],
-            disk=instance['disk'],
-            bandwidth=instance['bandwidth'],
-            price=self._get_size_price(size_id=instance['id']),
+            id=instance["id"],
+            name=instance["name"],
+            ram=instance["memory"],
+            disk=instance["disk"],
+            bandwidth=instance["bandwidth"],
+            price=self._get_size_price(size_id=instance["id"]),
             driver=self.connection.driver,
         )
 
     def list_instance_type(self, location=None):
-        return [self._instance_type_to_size(instance)
-                for name, instance in INSTANCE_TYPES.items()]
+        return [
+            self._instance_type_to_size(instance)
+            for name, instance in INSTANCE_TYPES.items()
+        ]
 
     def list_sizes(self, location=None):
         """
@@ -397,28 +401,28 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:  List of NodeSize objects
         :rtype:   ``list`` of :class:`NodeSize`
         """
-        account = self.connection.request('hosting.account.info').object
-        if account.get('rating_enabled'):
+        account = self.connection.request("hosting.account.info").object
+        if account.get("rating_enabled"):
             # This account use new rating model
             return self.list_instance_type(location)
         # Look for available shares, and return a list of share_definition
-        available_res = account['resources']['available']
+        available_res = account["resources"]["available"]
 
-        if available_res['shares'] == 0:
+        if available_res["shares"] == 0:
             return None
         else:
-            share_def = account['share_definition']
-            available_cores = available_res['cores']
+            share_def = account["share_definition"]
+            available_cores = available_res["cores"]
             # 0.75 core given when creating a server
             max_core = int(available_cores + 0.75)
             shares = []
-            if available_res['servers'] < 1:
+            if available_res["servers"] < 1:
                 # No server quota, no way
                 return shares
             for i in range(1, max_core + 1):
                 share = {id: i}
                 share_is_available = True
-                for k in ['memory', 'disk', 'bandwidth']:
+                for k in ["memory", "disk", "bandwidth"]:
                     if share_def[k] * i > available_res[k]:
                         # We run out for at least one resource inside
                         share_is_available = False
@@ -431,10 +435,7 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
     def _to_loc(self, loc):
         return NodeLocation(
-            id=loc['id'],
-            name=loc['dc_code'],
-            country=loc['country'],
-            driver=self
+            id=loc["id"], name=loc["dc_code"], country=loc["country"], driver=self
         )
 
     def list_locations(self):
@@ -444,8 +445,8 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return: List of NodeLocation objects
         :rtype: ``list`` of :class:`NodeLocation`
         """
-        res = self.connection.request('hosting.datacenter.list')
-        return [self._to_loc(l) for l in res.object]
+        res = self.connection.request("hosting.datacenter.list")
+        return [self._to_loc(loc) for loc in res.object]
 
     def list_volumes(self):
         """
@@ -454,7 +455,7 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return: A list of volume objects.
         :rtype: ``list`` of :class:`StorageVolume`
         """
-        res = self.connection.request('hosting.disk.list', {})
+        res = self.connection.request("hosting.disk.list", {})
         return self._to_volumes(res.object)
 
     def ex_get_volume(self, volume_id):
@@ -467,7 +468,7 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:  A StorageVolume object for the volume
         :rtype:   :class:`StorageVolume`
         """
-        res = self.connection.request('hosting.disk.info', volume_id)
+        res = self.connection.request("hosting.disk.info", volume_id)
         return self._to_volume(res.object)
 
     def create_volume(self, size, name, location=None, snapshot=None):
@@ -490,17 +491,18 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :rtype:   :class:`StorageVolume`
         """
         disk_param = {
-            'name': name,
-            'size': int(size),
-            'datacenter_id': int(location.id)
+            "name": name,
+            "size": int(size),
+            "datacenter_id": int(location.id),
         }
         if snapshot:
-            op = self.connection.request('hosting.disk.create_from',
-                                         disk_param, int(snapshot.id))
+            op = self.connection.request(
+                "hosting.disk.create_from", disk_param, int(snapshot.id)
+            )
         else:
-            op = self.connection.request('hosting.disk.create', disk_param)
-        if self._wait_operation(op.object['id']):
-            disk = self._volume_info(op.object['disk_id'])
+            op = self.connection.request("hosting.disk.create", disk_param)
+        if self._wait_operation(op.object["id"]):
+            disk = self._volume_info(op.object["disk_id"])
             return self._to_volume(disk)
         return None
 
@@ -520,9 +522,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:  True if successful
         :rtype:   ``bool``
         """
-        op = self.connection.request('hosting.vm.disk_attach',
-                                     int(node.id), int(volume.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request(
+            "hosting.vm.disk_attach", int(node.id), int(volume.id)
+        )
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -538,9 +541,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``bool``
         """
-        op = self.connection.request('hosting.vm.disk_detach',
-                                     int(node.id), int(volume.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request(
+            "hosting.vm.disk_detach", int(node.id), int(volume.id)
+        )
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -554,37 +558,31 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:  True if successful
         :rtype:   ``bool``
         """
-        op = self.connection.request('hosting.disk.delete', int(volume.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request("hosting.disk.delete", int(volume.id))
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
     def _to_iface(self, iface):
         ips = []
-        for ip in iface.get('ips', []):
+        for ip in iface.get("ips", []):
             new_ip = IPAddress(
-                ip['id'],
-                NODE_STATE_MAP.get(
-                    ip['state'],
-                    NodeState.UNKNOWN
-                ),
-                ip['ip'],
+                ip["id"],
+                NODE_STATE_MAP.get(ip["state"], NodeState.UNKNOWN),
+                ip["ip"],
                 self.connection.driver,
-                version=ip.get('version'),
-                extra={'reverse': ip['reverse']}
+                version=ip.get("version"),
+                extra={"reverse": ip["reverse"]},
             )
             ips.append(new_ip)
         return NetworkInterface(
-            iface['id'],
-            NODE_STATE_MAP.get(
-                iface['state'],
-                NodeState.UNKNOWN
-            ),
+            iface["id"],
+            NODE_STATE_MAP.get(iface["state"], NodeState.UNKNOWN),
             mac_address=None,
             driver=self.connection.driver,
             ips=ips,
-            node_id=iface.get('vm_id'),
-            extra={'bandwidth': iface['bandwidth']},
+            node_id=iface.get("vm_id"),
+            extra={"bandwidth": iface["bandwidth"]},
         )
 
     def _to_ifaces(self, ifaces):
@@ -596,24 +594,20 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``list`` of :class:`GandiNetworkInterface`
         """
-        ifaces = self.connection.request('hosting.iface.list').object
-        ips = self.connection.request('hosting.ip.list').object
+        ifaces = self.connection.request("hosting.iface.list").object
+        ips = self.connection.request("hosting.ip.list").object
         for iface in ifaces:
-            iface['ips'] = list(
-                filter(lambda i: i['iface_id'] == iface['id'], ips))
+            iface["ips"] = list(filter(lambda i: i["iface_id"] == iface["id"], ips))
         return self._to_ifaces(ifaces)
 
     def _to_disk(self, element):
         disk = Disk(
-            id=element['id'],
-            state=NODE_STATE_MAP.get(
-                element['state'],
-                NodeState.UNKNOWN
-            ),
-            name=element['name'],
+            id=element["id"],
+            state=NODE_STATE_MAP.get(element["state"], NodeState.UNKNOWN),
+            name=element["name"],
             driver=self.connection.driver,
-            size=element['size'],
-            extra={'can_snapshot': element['can_snapshot']}
+            size=element["size"],
+            extra={"can_snapshot": element["can_snapshot"]},
         )
         return disk
 
@@ -626,7 +620,7 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``list`` of :class:`GandiDisk`
         """
-        res = self.connection.request('hosting.disk.list', {})
+        res = self.connection.request("hosting.disk.list", {})
         return self._to_disks(res.object)
 
     def ex_node_attach_disk(self, node, disk):
@@ -641,9 +635,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``bool``
         """
-        op = self.connection.request('hosting.vm.disk_attach',
-                                     int(node.id), int(disk.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request(
+            "hosting.vm.disk_attach", int(node.id), int(disk.id)
+        )
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -659,9 +654,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``bool``
         """
-        op = self.connection.request('hosting.vm.disk_detach',
-                                     int(node.id), int(disk.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request(
+            "hosting.vm.disk_detach", int(node.id), int(disk.id)
+        )
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -677,9 +673,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``bool``
         """
-        op = self.connection.request('hosting.vm.iface_attach',
-                                     int(node.id), int(iface.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request(
+            "hosting.vm.iface_attach", int(node.id), int(iface.id)
+        )
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -695,9 +692,10 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``bool``
         """
-        op = self.connection.request('hosting.vm.iface_detach',
-                                     int(node.id), int(iface.id))
-        if self._wait_operation(op.object['id']):
+        op = self.connection.request(
+            "hosting.vm.iface_detach", int(node.id), int(iface.id)
+        )
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -713,17 +711,17 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: ``bool``
         """
-        if not disk.extra.get('can_snapshot'):
-            raise GandiException(1021, 'Disk %s can\'t snapshot' % disk.id)
+        if not disk.extra.get("can_snapshot"):
+            raise GandiException(1021, "Disk %s can't snapshot" % disk.id)
         if not name:
-            suffix = datetime.today().strftime('%Y%m%d')
-            name = 'snap_%s' % (suffix)
+            suffix = datetime.today().strftime("%Y%m%d")
+            name = "snap_%s" % (suffix)
         op = self.connection.request(
-            'hosting.disk.create_from',
-            {'name': name, 'type': 'snapshot', },
+            "hosting.disk.create_from",
+            {"name": name, "type": "snapshot"},
             int(disk.id),
         )
-        if self._wait_operation(op.object['id']):
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
@@ -744,22 +742,23 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         """
         params = {}
         if new_size:
-            params.update({'size': new_size})
+            params.update({"size": new_size})
         if new_name:
-            params.update({'name': new_name})
-        op = self.connection.request('hosting.disk.update',
-                                     int(disk.id),
-                                     params)
-        if self._wait_operation(op.object['id']):
+            params.update({"name": new_name})
+        op = self.connection.request("hosting.disk.update", int(disk.id), params)
+        if self._wait_operation(op.object["id"]):
             return True
         return False
 
     def _to_key_pair(self, data):
-        key_pair = KeyPair(name=data['name'],
-                           fingerprint=data['fingerprint'],
-                           public_key=data.get('value', None),
-                           private_key=data.get('privatekey', None),
-                           driver=self, extra={'id': data['id']})
+        key_pair = KeyPair(
+            name=data["name"],
+            fingerprint=data["fingerprint"],
+            public_key=data.get("value", None),
+            private_key=data.get("privatekey", None),
+            driver=self,
+            extra={"id": data["id"]},
+        )
         return key_pair
 
     def _to_key_pairs(self, data):
@@ -772,7 +771,7 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:   A list of key par objects.
         :rtype:   ``list`` of :class:`libcloud.compute.base.KeyPair`
         """
-        kps = self.connection.request('hosting.ssh.list').object
+        kps = self.connection.request("hosting.ssh.list").object
         return self._to_key_pairs(kps)
 
     def get_key_pair(self, name):
@@ -784,8 +783,8 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
 
         :rtype: :class:`.KeyPair`
         """
-        filter_params = {'name': name}
-        kps = self.connection.request('hosting.ssh.list', filter_params).object
+        filter_params = {"name": name}
+        kps = self.connection.request("hosting.ssh.list", filter_params).object
         return self._to_key_pair(kps[0])
 
     def import_key_pair_from_string(self, name, key_material):
@@ -801,8 +800,8 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return: Imported key pair object.
         :rtype: :class:`.KeyPair`
         """
-        params = {'name': name, 'value': key_material}
-        kp = self.connection.request('hosting.ssh.create', params).object
+        params = {"name": name, "value": key_material}
+        kp = self.connection.request("hosting.ssh.create", params).object
         return self._to_key_pair(kp)
 
     def delete_key_pair(self, key_pair):
@@ -815,7 +814,6 @@ class GandiNodeDriver(BaseGandiDriver, NodeDriver):
         :return:   True of False based on success of Keypair deletion
         :rtype:    ``bool``
         """
-        key_id = key_pair if isinstance(key_pair, int) \
-            else key_pair.extra['id']
-        success = self.connection.request('hosting.ssh.delete', key_id).object
+        key_id = key_pair if isinstance(key_pair, int) else key_pair.extra["id"]
+        success = self.connection.request("hosting.ssh.delete", key_id).object
         return success
