@@ -30,8 +30,10 @@ try:
     import paramiko
 
     have_paramiko = True
+
+    PARAMIKO_VERSION_TUPLE = tuple([int(x) for x in paramiko.__version__.split(".")])
 except ImportError:
-    pass
+    PARAMIKO_VERSION_TUPLE = ()
 
 # Depending on your version of Paramiko, it may cause a deprecation
 # warning on Python 2.6.
@@ -60,6 +62,29 @@ __all__ = [
 
 SUPPORTED_KEY_TYPES_URL = "https://libcloud.readthedocs.io/en/latest/compute/deployment.html#supported-private-ssh-key-types"  # NOQA
 
+# Set it to False to disable backward compatibility mode when running
+# paramiko >= 2.9.0. In backward compatibility mode we try to disable newer
+# SHA-2 based public key algorithms in case server returns auth error. This
+# way it works correctly with older OpenSSH servers which don't support those
+# algorithms aka the behavior is the same as with paramiko < 2.9.0.
+# In case users only talk to newer OpenSSH servers which support those
+# algorithms, they may want to disable this workaround.
+LIBCLOUD_PARAMIKO_SHA2_BACKWARD_COMPATIBILITY = os.environ.get(
+    "LIBCLOUD_PARAMIKO_SHA2_BACKWARD_COMPATIBILITY", "true"
+).lower() in [
+    "true",
+    "1",
+]
+
+SHA2_PUBKEY_NOT_SUPPORTED_AUTH_ERROR_MSG = """
+Received authentication error from the server. Disabling SHA-2 variants of RSA
+key verification algorithm for backward compatibility reasons and trying
+connecting again.
+
+You can disable this behavior by setting
+LIBCLOUD_PARAMIKO_SHA2_BACKWARD_COMPATIBILITY environment variable to "false".
+""".strip()
+
 
 class SSHCommandTimeoutError(Exception):
     """
@@ -377,7 +402,29 @@ class ParamikoSSHClient(BaseSSHClient):
 
         self.logger.debug("Connecting to server", extra=extra)
 
-        self.client.connect(**conninfo)
+        try:
+            self.client.connect(**conninfo)
+        except paramiko.ssh_exception.AuthenticationException as e:
+            # Special case to handle paramiko >= 2.9.0 which supports SHA-2
+            # variants of the RSA key verification algorithm which don't work
+            # with older OpenSSH server versions (e.g. default setup on Ubuntu
+            # 14.04).
+            # Sadly there is no way for us to catch and retry on more specific
+            # / granular exception.
+            # See https://www.paramiko.org/changelog.html for details.
+            if (
+                PARAMIKO_VERSION_TUPLE >= (2, 9, 0)
+                and LIBCLOUD_PARAMIKO_SHA2_BACKWARD_COMPATIBILITY
+            ):
+                self.logger.warn(SHA2_PUBKEY_NOT_SUPPORTED_AUTH_ERROR_MSG)
+
+                conninfo["disabled_algorithms"] = {
+                    "pubkeys": ["rsa-sha2-256", "rsa-sha2-512"]
+                }
+                self.client.connect(**conninfo)
+            else:
+                raise e
+
         return True
 
     def put(self, path, contents=None, chmod=None, mode="w"):
