@@ -30,8 +30,10 @@ from hashlib import sha256
 try:
     import fasteners
 except ImportError:
-    raise ImportError('Missing fasteners dependency, you can install it '
-                      'using pip: pip install fasteners')
+    raise ImportError(
+        "Missing fasteners dependency, you can install it "
+        "using pip: pip install fasteners"
+    )
 
 from libcloud.utils.files import read_in_chunks
 from libcloud.utils.files import exhaust_iterator
@@ -47,20 +49,35 @@ from libcloud.storage.types import ObjectError
 from libcloud.storage.types import ObjectDoesNotExistError
 from libcloud.storage.types import InvalidContainerNameError
 
-IGNORE_FOLDERS = ['.lock', '.hash']
+IGNORE_FOLDERS = [".lock", ".hash"]
+
+
+class NoOpLockLocalStorage(object):
+    def __init__(self, path, timeout=5):
+        self.path = path
+        self.lock_acquire_timeout = timeout
+
+    def __enter__(self):
+        return True
+
+    def __exit__(self, type, value, traceback):
+        return value
 
 
 class LockLocalStorage(object):
     """
-    A class to help in locking a local path before being updated
+    A class which locks a local path which is being updated. To correctly handle all the scenarios
+    use a thread based and IPC based lock.
     """
 
     def __init__(self, path, timeout=5):
         self.path = path
         self.lock_acquire_timeout = timeout
 
-        self.ipc_lock_path = os.path.join(tempfile.gettempdir(), "%s.lock" % (
-            sha256(path.encode("utf-8")).hexdigest()))
+        self.ipc_lock_path = os.path.join(
+            tempfile.gettempdir(),
+            "%s.lock" % (sha256(path.encode("utf-8")).hexdigest()),
+        )
 
         # NOTE: fasteners.InterProcess lock has no guarantees regards usage by
         # multiple threads in a single process which means we also need to
@@ -82,18 +99,18 @@ class LockLocalStorage(object):
                 break
 
         if not success:
-            raise LibcloudError("Failed to acquire thread lock for path %s "
-                                "in %s seconds" % (self.path,
-                                                   lock_acquire_timeout))
+            raise LibcloudError(
+                "Failed to acquire thread lock for path %s "
+                "in %s seconds" % (self.path, lock_acquire_timeout)
+            )
 
-        success = self.ipc_lock.acquire(blocking=True,
-                                        timeout=lock_acquire_timeout)
+        success = self.ipc_lock.acquire(blocking=True, timeout=lock_acquire_timeout)
 
         if not success:
-            raise LibcloudError("Failed to acquire IPC lock (%s) for path %s "
-                                "in %s seconds" %
-                                (self.ipc_lock_path, self.path,
-                                 lock_acquire_timeout))
+            raise LibcloudError(
+                "Failed to acquire IPC lock (%s) for path %s "
+                "in %s seconds" % (self.ipc_lock_path, self.path, lock_acquire_timeout)
+            )
 
     def __exit__(self, type, value, traceback):
         if self.thread_lock.locked():
@@ -114,22 +131,43 @@ class LocalStorageDriver(StorageDriver):
     """
 
     connectionCls = Connection
-    name = 'Local Storage'
-    website = 'http://example.com'
-    hash_type = 'md5'
+    name = "Local Storage"
+    website = "http://example.com"
+    hash_type = "md5"
 
-    def __init__(self, key, secret=None, secure=True, host=None, port=None,
-                 **kwargs):
+    def __init__(
+        self,
+        key,
+        secret=None,
+        secure=True,
+        host=None,
+        port=None,
+        ex_use_locking=True,
+        **kwargs,
+    ):
+        """
+        :param ex_use_locking: True if locking should be used when working with files. This value
+                               should almost always be left as-is and only changed to False when
+                               there is a specific need for it (e.g. for specific tests and micro
+                               benchmarks).
+        """
 
         # Use the key as the path to the storage
         self.base_path = key
 
         if not os.path.isdir(self.base_path):
-            raise LibcloudError('The base path is not a directory')
+            raise LibcloudError("The base path is not a directory")
 
-        super(LocalStorageDriver, self).__init__(key=key, secret=secret,
-                                                 secure=secure, host=host,
-                                                 port=port, **kwargs)
+        self._ex_use_locking = ex_use_locking
+
+        if self._ex_use_locking:
+            self._lock_cls = LockLocalStorage
+        else:
+            self._lock_cls = NoOpLockLocalStorage
+
+        super(LocalStorageDriver, self).__init__(
+            key=key, secret=secret, secure=secure, host=host, port=port, **kwargs
+        )
 
     def _make_path(self, path, ignore_existing=True):
         """
@@ -150,9 +188,10 @@ class LocalStorageDriver(StorageDriver):
         :type container_name: ``str``
         """
 
-        if '/' in container_name or '\\' in container_name:
-            raise InvalidContainerNameError(value=None, driver=self,
-                                            container_name=container_name)
+        if "/" in container_name or "\\" in container_name:
+            raise InvalidContainerNameError(
+                value=None, driver=self, container_name=container_name
+            )
 
     def _make_container(self, container_name):
         """
@@ -172,15 +211,16 @@ class LocalStorageDriver(StorageDriver):
         try:
             stat = os.stat(full_path)
             if not os.path.isdir(full_path):
-                raise OSError('Target path is not a directory')
+                raise OSError("Target path is not a directory")
         except OSError:
-            raise ContainerDoesNotExistError(value=None, driver=self,
-                                             container_name=container_name)
+            raise ContainerDoesNotExistError(
+                value=None, driver=self, container_name=container_name
+            )
 
         extra = {}
-        extra['creation_time'] = stat.st_ctime
-        extra['access_time'] = stat.st_atime
-        extra['modify_time'] = stat.st_mtime
+        extra["creation_time"] = stat.st_ctime
+        extra["access_time"] = stat.st_atime
+        extra["modify_time"] = stat.st_mtime
 
         return Container(name=container_name, extra=extra, driver=self)
 
@@ -206,24 +246,31 @@ class LocalStorageDriver(StorageDriver):
         try:
             stat = os.stat(full_path)
         except Exception:
-            raise ObjectDoesNotExistError(value=None, driver=self,
-                                          object_name=object_name)
+            raise ObjectDoesNotExistError(
+                value=None, driver=self, object_name=object_name
+            )
 
         # Make a hash for the file based on the metadata. We can safely
         # use only the mtime attribute here. If the file contents change,
         # the underlying file-system will change mtime
         data_hash = self._get_hash_function()
-        data_hash.update(u(stat.st_mtime).encode('ascii'))
+        data_hash.update(u(stat.st_mtime).encode("ascii"))
         data_hash = data_hash.hexdigest()
 
         extra = {}
-        extra['creation_time'] = stat.st_ctime
-        extra['access_time'] = stat.st_atime
-        extra['modify_time'] = stat.st_mtime
+        extra["creation_time"] = stat.st_ctime
+        extra["access_time"] = stat.st_atime
+        extra["modify_time"] = stat.st_mtime
 
-        return Object(name=object_name, size=stat.st_size, extra=extra,
-                      driver=self, container=container, hash=data_hash,
-                      meta_data=None)
+        return Object(
+            name=object_name,
+            size=stat.st_size,
+            extra=extra,
+            driver=self,
+            container=container,
+            hash=data_hash,
+            meta_data=None,
+        )
 
     def iterate_containers(self):
         """
@@ -257,8 +304,7 @@ class LocalStorageDriver(StorageDriver):
                 object_name = relpath(full_path, start=cpath)
                 yield self._make_object(container, object_name)
 
-    def iterate_container_objects(self, container, prefix=None,
-                                  ex_prefix=None):
+    def iterate_container_objects(self, container, prefix=None, ex_prefix=None):
         """
         Returns a generator of objects for the given container.
 
@@ -277,6 +323,8 @@ class LocalStorageDriver(StorageDriver):
         prefix = self._normalize_prefix_argument(prefix, ex_prefix)
 
         objects = self._get_objects(container)
+        objects = sorted(objects, key=lambda o: o.name)
+
         return self._filter_listed_container_objects(objects, prefix)
 
     def get_container(self, container_name):
@@ -307,8 +355,9 @@ class LocalStorageDriver(StorageDriver):
         path = os.path.join(self.base_path, container.name)
 
         if check and not os.path.isdir(path):
-            raise ContainerDoesNotExistError(value=None, driver=self,
-                                             container_name=container.name)
+            raise ContainerDoesNotExistError(
+                value=None, driver=self, container_name=container.name
+            )
 
         return path
 
@@ -352,7 +401,7 @@ class LocalStorageDriver(StorageDriver):
 
         path = self.get_container_cdn_url(container)
 
-        with LockLocalStorage(path):
+        with self._lock_cls(path):
             self._make_path(path)
 
         return True
@@ -368,19 +417,20 @@ class LocalStorageDriver(StorageDriver):
         """
         path = self.get_object_cdn_url(obj)
 
-        with LockLocalStorage(path):
+        with self._lock_cls(path):
             if os.path.exists(path):
                 return False
             try:
-                obj_file = open(path, 'w')
+                obj_file = open(path, "w")
                 obj_file.close()
             except Exception:
                 return False
 
         return True
 
-    def download_object(self, obj, destination_path, overwrite_existing=False,
-                        delete_on_failure=True):
+    def download_object(
+        self, obj, destination_path, overwrite_existing=False, delete_on_failure=True
+    ):
         """
         Download an object to the specified destination path.
 
@@ -408,7 +458,8 @@ class LocalStorageDriver(StorageDriver):
         file_path = self._get_obj_file_path(
             obj=obj,
             destination_path=destination_path,
-            overwrite_existing=overwrite_existing)
+            overwrite_existing=overwrite_existing,
+        )
 
         try:
             shutil.copy(obj_path, file_path)
@@ -436,54 +487,66 @@ class LocalStorageDriver(StorageDriver):
         :rtype: ``object``
         """
         path = self.get_object_cdn_url(obj)
-        with open(path, 'rb') as obj_file:
+        with open(path, "rb") as obj_file:
             for data in read_in_chunks(obj_file, chunk_size=chunk_size):
                 yield data
 
-    def download_object_range(self, obj, destination_path, start_bytes,
-                              end_bytes=None, overwrite_existing=False,
-                              delete_on_failure=True):
-        self._validate_start_and_end_bytes(start_bytes=start_bytes,
-                                           end_bytes=end_bytes)
+    def download_object_range(
+        self,
+        obj,
+        destination_path,
+        start_bytes,
+        end_bytes=None,
+        overwrite_existing=False,
+        delete_on_failure=True,
+    ):
+        self._validate_start_and_end_bytes(start_bytes=start_bytes, end_bytes=end_bytes)
 
         file_path = self._get_obj_file_path(
             obj=obj,
             destination_path=destination_path,
-            overwrite_existing=overwrite_existing)
+            overwrite_existing=overwrite_existing,
+        )
 
         iterator = self.download_object_range_as_stream(
-            obj=obj,
-            start_bytes=start_bytes,
-            end_bytes=end_bytes)
+            obj=obj, start_bytes=start_bytes, end_bytes=end_bytes
+        )
 
-        with open(file_path, 'wb') as fp:
+        with open(file_path, "wb") as fp:
             fp.write(exhaust_iterator(iterator))
 
         return True
 
-    def download_object_range_as_stream(self, obj, start_bytes, end_bytes=None,
-                                        chunk_size=None):
-        self._validate_start_and_end_bytes(start_bytes=start_bytes,
-                                           end_bytes=end_bytes)
+    def download_object_range_as_stream(
+        self, obj, start_bytes, end_bytes=None, chunk_size=None
+    ):
+        self._validate_start_and_end_bytes(start_bytes=start_bytes, end_bytes=end_bytes)
 
         path = self.get_object_cdn_url(obj)
-        with open(path, 'rb') as obj_file:
+        with open(path, "rb") as obj_file:
             file_size = len(obj_file.read())
 
             if end_bytes and end_bytes > file_size:
-                raise ValueError('end_bytes is larger than file size')
+                raise ValueError("end_bytes is larger than file size")
 
             if end_bytes is None:
                 read_bytes = (file_size - start_bytes) + 1
             else:
-                read_bytes = (end_bytes - start_bytes)
+                read_bytes = end_bytes - start_bytes
 
             obj_file.seek(start_bytes)
             data = obj_file.read(read_bytes)
             yield data
 
-    def upload_object(self, file_path, container, object_name, extra=None,
-                      verify_hash=True, headers=None):
+    def upload_object(
+        self,
+        file_path,
+        container,
+        object_name,
+        extra=None,
+        verify_hash=True,
+        headers=None,
+    ):
         """
         Upload an object currently located on a disk.
 
@@ -514,16 +577,16 @@ class LocalStorageDriver(StorageDriver):
 
         self._make_path(base_path)
 
-        with LockLocalStorage(obj_path):
+        with self._lock_cls(obj_path):
             shutil.copy(file_path, obj_path)
 
-        os.chmod(obj_path, int('664', 8))
+        os.chmod(obj_path, int("664", 8))
 
         return self._make_object(container, object_name)
 
-    def upload_object_via_stream(self, iterator, container,
-                                 object_name,
-                                 extra=None, headers=None):
+    def upload_object_via_stream(
+        self, iterator, container, object_name, extra=None, headers=None
+    ):
         """
         Upload an object using an iterator.
 
@@ -565,11 +628,10 @@ class LocalStorageDriver(StorageDriver):
         obj_path = os.path.join(path, object_name)
         base_path = os.path.dirname(obj_path)
         self._make_path(base_path)
-        with LockLocalStorage(obj_path):
-            with open(obj_path, 'wb') as obj_file:
-                for data in iterator:
-                    obj_file.write(data)
-        os.chmod(obj_path, int('664', 8))
+        with self._lock_cls(obj_path), open(obj_path, "wb") as obj_file:
+            for data in iterator:
+                obj_file.write(data)
+        os.chmod(obj_path, int("664", 8))
         return self._make_object(container, object_name)
 
     def delete_object(self, obj):
@@ -585,7 +647,7 @@ class LocalStorageDriver(StorageDriver):
 
         path = self.get_object_cdn_url(obj)
 
-        with LockLocalStorage(path):
+        with self._lock_cls(path):
             try:
                 os.unlink(path)
             except Exception:
@@ -628,17 +690,20 @@ class LocalStorageDriver(StorageDriver):
         except OSError as exp:
             if exp.errno == errno.EEXIST:
                 raise ContainerAlreadyExistsError(
-                    value='Container with this name already exists. The name '
-                          'must be unique among all the containers in the '
-                          'system',
-                    container_name=container_name, driver=self)
+                    value="Container with this name already exists. The name "
+                    "must be unique among all the containers in the "
+                    "system",
+                    container_name=container_name,
+                    driver=self,
+                )
             else:
                 raise LibcloudError(
-                    'Error creating container %s' % container_name,
-                    driver=self)
+                    "Error creating container %s" % container_name, driver=self
+                )
         except Exception:
             raise LibcloudError(
-                'Error creating container %s' % container_name, driver=self)
+                "Error creating container %s" % container_name, driver=self
+            )
 
         return self._make_container(container_name)
 
@@ -655,13 +720,15 @@ class LocalStorageDriver(StorageDriver):
 
         # Check if there are any objects inside this
         for obj in self._get_objects(container):
-            raise ContainerIsNotEmptyError(value='Container is not empty',
-                                           container_name=container.name,
-                                           driver=self)
+            raise ContainerIsNotEmptyError(
+                value="Container is not empty",
+                container_name=container.name,
+                driver=self,
+            )
 
         path = self.get_container_cdn_url(container, check=True)
 
-        with LockLocalStorage(path):
+        with self._lock_cls(path):
             try:
                 shutil.rmtree(path)
             except Exception:
@@ -669,15 +736,14 @@ class LocalStorageDriver(StorageDriver):
 
         return True
 
-    def _get_obj_file_path(self, obj, destination_path,
-                           overwrite_existing=False):
+    def _get_obj_file_path(self, obj, destination_path, overwrite_existing=False):
         # type: (Object, str, bool) -> str
         base_name = os.path.basename(destination_path)
 
         if not base_name and not os.path.exists(destination_path):
             raise LibcloudError(
-                value='Path %s does not exist' % (destination_path),
-                driver=self)
+                value="Path %s does not exist" % (destination_path), driver=self
+            )
 
         if not base_name:
             file_path = os.path.join(destination_path, obj.name)
@@ -686,8 +752,9 @@ class LocalStorageDriver(StorageDriver):
 
         if os.path.exists(file_path) and not overwrite_existing:
             raise LibcloudError(
-                value='File %s already exists, but ' % (file_path) +
-                'overwrite_existing=False',
-                driver=self)
+                value="File %s already exists, but " % (file_path)
+                + "overwrite_existing=False",
+                driver=self,
+            )
 
         return file_path
